@@ -861,6 +861,10 @@ public:
   static bool
   IsScrollGrabAllowed(JSContext*, JSObject*);
 
+  virtual inline void ResultForDialogSubmit(nsAString& aResult) {
+    GetAttr(kNameSpaceID_None, nsGkAtoms::value, aResult);
+  }
+
 protected:
   /**
    * Add/remove this element to the documents name cache
