@@ -1,5 +1,4 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 et cindent: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -27,25 +26,16 @@ static FFmpegLibWrapper sLibAV;
 
 static const char* sLibs[] = {
 #if defined(XP_DARWIN)
+  "libavcodec.61.dylib",
+  "libavcodec.60.dylib",
   "libavcodec.59.dylib",
   "libavcodec.58.dylib",
-  "libavcodec.57.dylib",
-  "libavcodec.56.dylib",
-  "libavcodec.55.dylib",
-  "libavcodec.54.dylib",
-  "libavcodec.53.dylib",
 #else
+  "libavcodec.so.61",
+  "libavcodec.so.60",
   "libavcodec.so.59",
   "libavcodec.so.58",
-  "libavcodec-ffmpeg.so.59",
   "libavcodec-ffmpeg.so.58",
-  "libavcodec-ffmpeg.so.57",
-  "libavcodec-ffmpeg.so.56",
-  "libavcodec.so.57",
-  "libavcodec.so.56",
-  "libavcodec.so.55",
-  "libavcodec.so.54",
-  "libavcodec.so.53",
 #endif
 };
 
@@ -82,38 +72,13 @@ FFmpegRuntimeLinker::Init()
             sLinkStatusLibraryName = lib;
           }
           break;
-        case FFmpegLibWrapper::LinkResult::CannotUseLibAV57:
-          if (sLinkStatus > LinkStatus_UNUSABLE_LIBAV57) {
-            sLinkStatus = LinkStatus_UNUSABLE_LIBAV57;
-            sLinkStatusLibraryName = lib;
-          }
-          break;
-        case FFmpegLibWrapper::LinkResult::BlockedOldLibAVVersion:
-          if (sLinkStatus > LinkStatus_OBSOLETE_LIBAV) {
-            sLinkStatus = LinkStatus_OBSOLETE_LIBAV;
-            sLinkStatusLibraryName = lib;
-          }
-          break;
-        case FFmpegLibWrapper::LinkResult::UnknownFutureLibAVVersion:
-        case FFmpegLibWrapper::LinkResult::MissingLibAVFunction:
-          if (sLinkStatus > LinkStatus_INVALID_LIBAV_CANDIDATE) {
-            sLinkStatus = LinkStatus_INVALID_LIBAV_CANDIDATE;
-            sLinkStatusLibraryName = lib;
-          }
-          break;
-        case FFmpegLibWrapper::LinkResult::UnknownFutureFFMpegVersion:
+	    case FFmpegLibWrapper::LinkResult::UnknownFFMpegVersion:
         case FFmpegLibWrapper::LinkResult::MissingFFMpegFunction:
           if (sLinkStatus > LinkStatus_INVALID_FFMPEG_CANDIDATE) {
             sLinkStatus = LinkStatus_INVALID_FFMPEG_CANDIDATE;
             sLinkStatusLibraryName = lib;
           }
           break;
-        case FFmpegLibWrapper::LinkResult::UnknownOlderFFMpegVersion:
-          if (sLinkStatus > LinkStatus_OBSOLETE_FFMPEG) {
-            sLinkStatus = LinkStatus_OBSOLETE_FFMPEG;
-            sLinkStatusLibraryName = lib;
-          }
-          break;
       }
     }
   }
@@ -135,13 +100,10 @@ FFmpegRuntimeLinker::CreateDecoderModule()
   }
   RefPtr<PlatformDecoderModule> module;
   switch (sLibAV.mVersion) {
-    case 53: module = FFmpegDecoderModule<53>::Create(&sLibAV); break;
-    case 54: module = FFmpegDecoderModule<54>::Create(&sLibAV); break;
-    case 55:
-    case 56: module = FFmpegDecoderModule<55>::Create(&sLibAV); break;
-    case 57: module = FFmpegDecoderModule<57>::Create(&sLibAV); break;
     case 58: module = FFmpegDecoderModule<58>::Create(&sLibAV); break;
     case 59: module = FFmpegDecoderModule<59>::Create(&sLibAV); break;
+    case 60: module = FFmpegDecoderModule<60>::Create(&sLibAV); break;
+	case 61: module = FFmpegDecoderModule<61>::Create(&sLibAV); break;
     default: module = nullptr;
   }
   return module.forget();
@@ -157,14 +119,6 @@ FFmpegRuntimeLinker::LinkStatusString()
       return "Libavcodec linking succeeded";
     case LinkStatus_INVALID_FFMPEG_CANDIDATE:
       return "Invalid FFMpeg libavcodec candidate";
-    case LinkStatus_UNUSABLE_LIBAV57:
-      return "Unusable LibAV's libavcodec 57";
-    case LinkStatus_INVALID_LIBAV_CANDIDATE:
-      return "Invalid LibAV libavcodec candidate";
-    case LinkStatus_OBSOLETE_FFMPEG:
-      return "Obsolete FFMpeg libavcodec candidate";
-    case LinkStatus_OBSOLETE_LIBAV:
-      return "Obsolete LibAV libavcodec candidate";
     case LinkStatus_INVALID_CANDIDATE:
       return "Invalid libavcodec candidate";
     case LinkStatus_NOT_FOUND:
